from django import forms
from .models import FileData, ImageData

class FileUpload(forms.ModelForm):
	class Meta:
		model = FileData
		fields = ('file', 'description')
		def save(self):
			file = super(FileUpload, self).save()
			return file


class ImageUpload(forms.ModelForm):
	class Meta:
		model = ImageData
		fields = ('file', 'description')
		def save(self):
			image = super(ImageUpload, self).save()
			return image


# Generated by Django 4.2 on 2023-05-01 12:37

from django.db import migrations, models
import django.db.models.deletion
import django_bleach.models
from django.conf import settings


class Migration(migrations.Migration):

	initial = True

	dependencies = [
		('auth', '__first__'),
		migrations.swappable_dependency(settings.AUTH_USER_MODEL),
		('sites', '__first__'),
	]

	operations = [
		migrations.CreateModel(
			name='FileData',
			fields=[
				('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('description', models.CharField(blank=True, max_length=512)),
				('uploaded_at', models.DateTimeField(auto_now_add=True)),
				('file', models.FileField(upload_to='')),
			],
			options={
				'abstract': False,
			},
		),
		migrations.CreateModel(
			name='ImageData',
			fields=[
				('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('description', models.CharField(blank=True, max_length=512)),
				('uploaded_at', models.DateTimeField(auto_now_add=True)),
				('file', models.ImageField(height_field='height', upload_to='', width_field='width')),
				('height', models.PositiveIntegerField(default=0)),
				('width', models.PositiveIntegerField(default=0)),
			],
			options={
				'abstract': False,
			},
		),
		migrations.CreateModel(
			name='Page',
			fields=[
				('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('key', models.CharField(max_length=64)),
				('title', models.CharField(max_length=128)),
				('title_en', models.CharField(max_length=128, null=True)),
				('title_es', models.CharField(max_length=128, null=True)),
				('description', models.CharField(blank=True, max_length=256)),
				('description_en', models.CharField(blank=True, max_length=256, null=True)),
				('description_es', models.CharField(blank=True, max_length=256, null=True)),
				('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
			],
			options={
				'unique_together': {('site', 'key')},
			},
		),
		migrations.CreateModel(
			name='Text',
			fields=[
				('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('key', models.CharField(max_length=64)),
				('content', django_bleach.models.BleachField()),
				('content_en', django_bleach.models.BleachField(null=True)),
				('content_es', django_bleach.models.BleachField(null=True)),
				('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_cms.page')),
			],
			options={
				'abstract': False,
				'unique_together': {('page', 'key')},
			},
		),
		migrations.CreateModel(
			name='Image',
			fields=[
				('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('key', models.CharField(max_length=64)),
				('content', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='drf_cms.imagedata')),
				('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_cms.page')),
			],
			options={
				'abstract': False,
				'unique_together': {('page', 'key')},
			},
		),
		migrations.CreateModel(
			name='File',
			fields=[
				('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
				('key', models.CharField(max_length=64)),
				('content', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='file_data', to='drf_cms.filedata')),
				('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_cms.page')),
			],
			options={
				'abstract': False,
				'unique_together': {('page', 'key')},
			},
		),
	]



from rest_framework import serializers
from .models import Page, Text, FileData, ImageData


class TextSerializer(serializers.ModelSerializer):
    key = serializers.CharField(read_only=True)

    class Meta:
        model = Text
        fields = ('content', 'key')


class PageSerializer(serializers.ModelSerializer):
    texts = serializers.SerializerMethodField(read_only=True)

    class Meta:
        model = Page
        fields = ('key', 'title', 'description', 'texts')

    def get_texts(self, instance):
        texts = instance.text_set.all()
        result = {}
        for text in texts:
            result[text.key] = text.content
        return result


class FileDataSerializer(serializers.ModelSerializer):
	file = serializers.FileField()
	extra = serializers.SerializerMethodField()

	class Meta:
		model = FileData
		fields = ('file', 'id', 'description', 'extra')

	def get_extra(self, instance):
		if not instance.file:
			return None
		if (pathlib.Path(instance.file.url).suffix.lower() == '.mp4'):
			url = urlparse(instance.file.url)
			filename = pathlib.Path(url.path).stem
			base_url = 'https://{}.s3.amazonaws.com/{}'.format(
				settings.AWS_STORAGE_VIDEO_BUCKET_NAME, filename)
			file = {
				'hls': '{}/HLS/{}.m3u8'.format(base_url, filename),
				'thumbnail': '{}/Thumbnails/{}.0000004.jpg'.format(
								base_url, filename),
				'thumbnail_fallback': '{}/Thumbnails/{}.0000002.jpg'.format(
								base_url, filename)
			}
			return file
		return None


class ImageSerializer(serializers.ModelSerializer):
	class Meta:
		model = ImageData
		fields = ('file', 'id', 'description')


from modeltranslation.translator import register, TranslationOptions
from .models import Text, Page


@register(Text)
class TextTranslationOptions(TranslationOptions):
    fields = ('content', )


@register(Page)
class PageTranslationOptions(TranslationOptions):
    fields = ('title', 'description', )


from django.urls import path
from .views import *

urlpatterns = [
    path('pages/<str:page_key>/texts/', TextView.as_view(), name='text-list'),
    path('pages/<str:page_key>/texts/<str:key>/', TextView.as_view(), name='text-obj'),
    path('pages/<str:key>/', PageView.as_view(), name='page-list'),
    path('pages/', PageView.as_view(), name='page-obj'),
    path('files', FileUploadView.as_view(), name = 'file_upload'),
	path('files/signed', SignedUrlView.as_view(), name = 'get_signed'),
    path('images/upload', ImageUploadView.as_view(), name = 'image_upload'),
]



from rest_framework import permissions
from .shortcuts import get_current_site

class IsOwnerOrReadOnly(permissions.BasePermission):

	def has_permission(self, request, view):
		if request.method in permissions.SAFE_METHODS:
			return True

		site = get_current_site(request)
		return (request.user.is_authenticated and request.user.is_staff)

	def has_object_permission(self, request, view, obj):	
		if request.method in permissions.SAFE_METHODS:
			return True

		site = get_current_site(request)
		return site and request.user.is_staff and \
			((hasattr(obj, 'site') and obj.site == site) \
				or \
			(hasattr(obj, 'page') and obj.page.site == site))



from django.contrib import admin
from .models import *

@admin.register(Page)
class PageAdmin(admin.ModelAdmin):
    model = Page
    list_filter = ('site',)

@admin.register(Text)
class TextAdmin(admin.ModelAdmin):
    model = Text
    list_display = ('key', 'page', )
    list_filter = ('page', 'page__site')

@admin.register(FileData)
class FileDataAdmin(admin.ModelAdmin):
	list_display = ('id', 'file', 'description', 'uploaded_at',)
	readonly_fields=['uploaded_at']
	search_fields = ['description']
	list_filter = ('description',)
	ordering = ('-uploaded_at', )

@admin.register(ImageData)
class ImageDataAdmin(admin.ModelAdmin):
	list_display = ('id', 'file', 'description', 'uploaded_at',)
	readonly_fields=['uploaded_at']
	search_fields = ['description']
	list_filter = ('description',)
	ordering = ('-uploaded_at', )



from django.shortcuts import render, redirect
from django.conf import settings
from rest_framework import generics, mixins, status
from rest_framework.parsers import FileUploadParser
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response
from rest_framework.views import APIView
from .models import Text, FileData, ImageData
from .forms import ImageUpload
from .serializers import *
from .permissions import IsOwnerOrReadOnly
from .shortcuts import get_current_site
from .storage_backends import create_presigned_post


class TextView(
		mixins.ListModelMixin, 
		mixins.RetrieveModelMixin,
		mixins.UpdateModelMixin,
		generics.GenericAPIView):
	serializer_class = TextSerializer
	permission_classes = [IsOwnerOrReadOnly]
	lookup_field = 'key'

	def get_queryset(self):
		page_key = self.kwargs.get('page_key')
		site = get_current_site(self.request)
		return Text.objects.filter(page__key=page_key, page__site=site)

	def get(self, request, *args, **kwargs):
		key = self.kwargs.get('key')
		if (key):
			return self.retrieve(request, *args, **kwargs)
		return self.list(request, *args, **kwargs)

	def patch(self, request, *args, **kwargs):
		return self.partial_update(request, *args, **kwargs)



class PageView(
		mixins.ListModelMixin,
		mixins.RetrieveModelMixin,
		generics.GenericAPIView):
	serializer_class = PageSerializer
	permission_classes = [IsOwnerOrReadOnly]
	lookup_field = 'key'

	def get_queryset(self):
		site = get_current_site(self.request)
		return Page.objects.filter(site=site)

	def get(self, request, *args, **kwargs):
		key = self.kwargs.get('key')
		if (key):
			return self.retrieve(request, *args, **kwargs)
		return self.list(request, *args, **kwargs)


class FileUploadView(APIView):
    permission_classes = [IsAdminUser]
    parser_class = (FileUploadParser,)

    def post(self, request, *args, **kwargs):
      file = None
      url = request.data.get('url', None)
      description = request.data.get('description', '')
      if url:
        file = FileData.objects.create()
        file.file.name = url.lower()
        file.description = description
        file.save()
      else:
        file_serializer = FileDataSerializer(data=request.data)

        if file_serializer.is_valid():
            file = file_serializer.save()
          
      if file:
          print(FileDataSerializer(file).data)
          return Response(FileDataSerializer(file).data, status=status.HTTP_201_CREATED)
      else:
          return Response(file_serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class SignedUrlView(APIView):
    permission_classes = [IsAdminUser]
    def get(self, request, *args, **kwargs):
      filename = request.GET.get('filename', None)
      if not filename:
        return Response(
          {'message': 'missing url param filename'}, 
          status=status.HTTP_400_BAD_REQUEST)
      
      response = create_presigned_post(
        settings.AWS_STORAGE_BUCKET_NAME, 
        filename)

      if not response:
        return Response(
          {'message': 'Error signing url'}, 
          status=status.HTTP_503_SERVICE_UNAVAILABLE)
      return Response(response, status=status.HTTP_200_OK)


class ImageUploadView(APIView):
    parser_class = (FileUploadParser,)

    def post(self, request, *args, **kwargs):

      file_serializer = ImageDataSerializer(data=request.data)

      if file_serializer.is_valid():
        file_serializer.save()
        return Response(file_serializer.data, status=status.HTTP_201_CREATED)
      else:
        return Response(file_serializer.errors, status=status.HTTP_400_BAD_REQUEST)




from storages.backends.s3boto3 import S3Boto3Storage 
import boto3
from botocore.config import Config
from botocore.exceptions import ClientError

import random
import os


class MediaStorage(S3Boto3Storage):    
	location = 'media'
	file_overwrite = False


def create_presigned_post(bucket_name, object_name,
		fields=None, conditions=None, expiration=3600):
	"""Generate a presigned URL S3 POST request to upload a file

	:param bucket_name: string
	:param object_name: string
	:param fields: Dictionary of prefilled form fields
	:param conditions: List of conditions to include in the policy
	:param expiration: Time in seconds for the presigned URL to remain valid
	:return: Dictionary with the following keys:
		url: URL to post to
		fields: Dictionary of form fields and values to submit with the POST
	:return: None if error.
	"""
	filename, file_extension = os.path.splitext(object_name)
	object_name = '{}_{}{}'.format(filename, random.randint(10000, 100000), file_extension)

	s3_client = boto3.client('s3', config=Config(
		region_name = 'eu-west-3',
		signature_version = 's3v4',
		retries = {
			'max_attempts': 10,
			'mode': 'standard'
		}
	))
	try:
		response = s3_client.generate_presigned_post(
			bucket_name,
			'{}/{}'.format(MediaStorage.location, object_name),
			Fields=fields,
			Conditions=conditions,
			ExpiresIn=expiration)
		response['filename'] = object_name
	except ClientError as e:
		return None

	return response


from django.db import models
from django_bleach.models import BleachField
from django.contrib.sites.models import Site


class Page(models.Model):
	site = models.ForeignKey(Site, null=False, on_delete=models.CASCADE)
	key = models.CharField(null=False, max_length=64)
	title = models.CharField(null=False, max_length=128)
	description = models.CharField(blank=True, max_length=256)

	class Meta:
		unique_together = ('site', 'key')

	def __str__(self):
		return '{} - {}'.format(self.key, self.site.domain)


class ContentMixin(models.Model):
	page = models.ForeignKey(Page, null=False, on_delete=models.CASCADE)
	key = models.CharField(null=False, max_length=64)

	class Meta:
		unique_together = ('page', 'key')
		abstract = True

	def __str__(self):
		return self.key


class MetadataMixin(models.Model):
	description = models.CharField(blank=True, max_length=512)
	uploaded_at =  models.DateTimeField(auto_now_add=True)

	class Meta:
		abstract = True

	def __str__(self):
		return self.description


class ImageData(MetadataMixin):
	file = models.ImageField(height_field='height', width_field='width')
	height = models.PositiveIntegerField(default=0)
	width = models.PositiveIntegerField(default=0)


class FileData(MetadataMixin):
	file = models.FileField()


class Text(ContentMixin):
	content = BleachField(null=False, allowed_tags=['a', 'p', 'i', 'b', 'u'], allowed_attributes=['href'])


class Image(ContentMixin):
	content = models.ForeignKey(ImageData, null=False, on_delete=models.PROTECT)


class File(ContentMixin):
	content = models.ForeignKey(FileData, null=False, on_delete=models.PROTECT, related_name='file_data')
